/*
 * Copyright 2010 Google Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.google.gwt.sample.expenses.gwt.request;

import com.google.gwt.requestfactory.shared.RecordListRequest;
import com.google.gwt.requestfactory.shared.RecordRequest;
import com.google.gwt.requestfactory.shared.RequestFactory;
import com.google.gwt.requestfactory.shared.ServerOperation;
import com.google.gwt.valuestore.shared.PropertyReference;

/**
 * "API Generated" request selector interface implemented by objects that give
 * client access to the methods of
 * {@link com.google.gwt.sample.expenses.server.domain.Employee}.
 * <p>
 * IRL this class will be generated by a JPA-savvy tool run before compilation.
 */
public interface EmployeeRequest {

  /**
   * Defines the server operations that handle these requests.
   */
  public enum ServerOperations implements RequestFactory.RequestDefinition {
    COUNT_EMPLOYEES {
      public String getDomainMethodName() {
        return "countEmployees";
      }

      public Class<?> getReturnType() {
        return Long.class;
      }

      public boolean isReturnTypeList() {
        return false;
      }
    },

    COUNT_EMPLOYEES_BY_DEPARTMENT {
      public String getDomainMethodName() {
        return "countEmployeesByDepartment";
      }

      public Class<?>[] getParameterTypes() {
        return new Class[]{java.lang.String.class};
      }

      public Class<?> getReturnType() {
        return Long.class;
      }

      public boolean isReturnTypeList() {
        return false;
      }
    },

    FIND_ALL_EMPLOYEES {
      public String getDomainMethodName() {
        return "findAllEmployees";
      }
    },

    FIND_EMPLOYEE {
      public String getDomainMethodName() {
        return "findEmployee";
      }

      public Class<?>[] getParameterTypes() {
        return new Class[] {Long.class};
      }

      public boolean isReturnTypeList() {
        return false;
      }
    },

    FIND_EMPLOYEE_ENTRIES {
      public String getDomainMethodName() {
        return "findEmployeeEntries";
      }

      public Class<?>[] getParameterTypes() {
        return new Class[]{int.class, int.class};
      }
    },

    FIND_EMPLOYEE_ENTRIES_BY_DEPARTMENT {
      public String getDomainMethodName() {
        return "findEmployeeEntriesByDepartment";
      }

      public Class<?>[] getParameterTypes() {
        return new Class[]{java.lang.String.class, int.class, int.class};
      }
    };

    public String getDomainClassName() {
      return "com.google.gwt.sample.expenses.server.domain.Employee";
    }

    public Class<?>[] getParameterTypes() {
      return null;
    }

    public Class<?> getReturnType() {
      return EmployeeRecord.class;
    }

    public boolean isReturnTypeList() {
      return true;
    }
  }

  /**
   * @return a request object
   */
  @ServerOperation("COUNT_EMPLOYEES")
  RequestFactory.RequestObject<Long> countEmployees();

  /**
   * @return a request object
   */
  @ServerOperation("COUNT_EMPLOYEES_BY_DEPARTMENT")
  RequestFactory.RequestObject<Long> countEmployeesByDepartment(
      String department);

  /**
   * @return a request object
   */
  @ServerOperation("FIND_ALL_EMPLOYEES")
  RecordListRequest<EmployeeRecord> findAllEmployees();

  /**
   * @return a request object
   */
  @ServerOperation("FIND_EMPLOYEE")
  RecordRequest<EmployeeRecord> findEmployee(PropertyReference<String> id);

  /**
   * @return a request object
   */
  @ServerOperation("FIND_EMPLOYEE_ENTRIES")
  RecordListRequest<EmployeeRecord> findEmployeeEntries(int firstResult,
      int maxResults);

  /**
   * @return a request object
   */
  @ServerOperation("FIND_EMPLOYEE_ENTRIES_BY_DEPARTMENT")
  RecordListRequest<EmployeeRecord> findEmployeeEntriesByDepartment(
      String department, int firstResult, int maxResults);
}
