/*
 * Copyright 2010 Google Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.google.gwt.sample.expenses.gwt.request;

import com.google.gwt.requestfactory.shared.RecordListRequest;
import com.google.gwt.requestfactory.shared.RequestFactory;
import com.google.gwt.requestfactory.shared.ServerOperation;
import com.google.gwt.valuestore.shared.PropertyReference;
import com.google.gwt.valuestore.shared.Record;

/**
 * "API Generated" request selector interface implemented by objects that give
 * client access to the methods of
 * {@link com.google.gwt.sample.expenses.server.domain.Expense}.
 * <p>
 * IRL this class will be generated by a JPA-savvy tool run before compilation.
 */
public interface ExpenseRequest {

  /**
   * Defines the server operations that handle these requests.
   */
  public enum ServerOperations implements RequestFactory.RequestDefinition {
    FIND_ALL_EXPENSES {
      public String getDomainMethodName() {
        return "findAllExpenses";
      }
    },

    FIND_EXPENSE {
      public String getDomainMethodName() {
        return "findListOfOneExpense";
      }

      public Class<?>[] getParameterTypes() {
        return new Class[] {java.lang.Long.class};
      }
    },

    FIND_EXPENSES_BY_REPORT {
      public String getDomainMethodName() {
        return "findExpensesByReport";
      }

      public Class<?>[] getParameterTypes() {
        return new Class[] {java.lang.Long.class};
      }
    };

    public String getDomainClassName() {
      return "com.google.gwt.sample.expenses.server.domain.Expense";
    }

    public Class<?>[] getParameterTypes() {
      return null;
    }

    public Class<? extends Record> getReturnType() {
      return ExpenseRecord.class;
    }
    
    public boolean isReturnTypeList() {
      return true;
    }
  }

  /**
   * @return a request object
   */
  @ServerOperation("FIND_ALL_EXPENSES")
  RecordListRequest<ExpenseRecord> findAllExpenses();

  /**
   * @return a request object
   */
  @ServerOperation("FIND_EXPENSE")
  RecordListRequest<ExpenseRecord> findExpense(
      PropertyReference<String> id);

  /**
   * @return a request object
   */
  @ServerOperation("FIND_EXPENSES_BY_REPORT")
  RecordListRequest<ExpenseRecord> findExpensesByReport(
      PropertyReference<String> reportId);
}
