/*
 * Copyright 2010 Google Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.google.gwt.sample.expenses.gwt.request;

import com.google.gwt.requestfactory.shared.RecordListRequest;
import com.google.gwt.requestfactory.shared.RecordRequest;
import com.google.gwt.requestfactory.shared.RequestFactory;
import com.google.gwt.requestfactory.shared.ServerOperation;
import com.google.gwt.valuestore.shared.PropertyReference;

/**
 * "API Generated" request selector interface implemented by objects that give
 * client access to the methods of
 * {@link com.google.gwt.sample.expenses.server.domain.Report}.
 * <p>
 * IRL this class will be generated by a JPA-savvy tool run before compilation.
 */
public interface ReportRequest {

  /**
   * Defines the server operations that handle these requests.
   */
  public enum ServerOperations implements RequestFactory.RequestDefinition {
    COUNT_REPORTS {
      public String getDomainMethodName() {
        return "countReports";
      }

      public Class<?> getReturnType() {
        return long.class;
      }

      public boolean isReturnTypeList() {
        return false;
      }
    },

    COUNT_REPORTS_BY_SEARCH {
      public String getDomainMethodName() {
        return "countReportsBySearch";
      }

      public Class<?>[] getParameterTypes() {
        return new Class[] {java.lang.Long.class, java.lang.String.class,
            java.lang.String.class};
      }

      public Class<?> getReturnType() {
        return long.class;
      }

      public boolean isReturnTypeList() {
        return false;
      }
    },

    FIND_ALL_REPORTS {
      public String getDomainMethodName() {
        return "findAllReports";
      }
    },

    FIND_REPORT {
      public String getDomainMethodName() {
        return "findReport";
      }

      public Class<?>[] getParameterTypes() {
        return new Class[] {java.lang.Long.class};
      }

      public boolean isReturnTypeList() {
        return false;
      }
    },

    FIND_REPORTS_BY_EMPLOYEE {
      public String getDomainMethodName() {
        return "findReportsByEmployee";
      }

      public Class<?>[] getParameterTypes() {
        return new Class[] {java.lang.Long.class};
      }
    },

    FIND_REPORT_ENTRIES {
      public String getDomainMethodName() {
        return "findReportEntries";
      }

      public Class<?>[] getParameterTypes() {
        return new Class[] {int.class, int.class};
      }
    },

    FIND_REPORT_ENTRIES_BY_SEARCH {
      public String getDomainMethodName() {
        return "findReportEntriesBySearch";
      }

      public Class<?>[] getParameterTypes() {
        return new Class[]{
            java.lang.Long.class, java.lang.String.class,
            java.lang.String.class, java.lang.String.class, int.class,
            int.class};
      }
    };

    public String getDomainClassName() {
      return "com.google.gwt.sample.expenses.server.domain.Report";
    }

    public Class<?>[] getParameterTypes() {
      return null;
    }

    public Class<?> getReturnType() {
      return ReportRecord.class;
    }

    public boolean isReturnTypeList() {
      return true;
    }
  }

  /**
   * @return a request object
   */
  @ServerOperation("COUNT_REPORTS")
  RequestFactory.RequestObject<Long> countReports();

  /**
   * @return a request object
   */
  @ServerOperation("COUNT_REPORTS_BY_SEARCH")
  RequestFactory.RequestObject<Long> countReportsBySearch(Long employeeId,
      String department, String startsWith);

  /**
   * @return a request object
   */
  @ServerOperation("FIND_ALL_REPORTS")
  RecordListRequest<ReportRecord> findAllReports();

  /**
   * @return a request object
   */
  @ServerOperation("FIND_REPORT")
  RecordRequest<ReportRecord> findReport(PropertyReference<String> id);

  /**
   * @return a request object
   */
  @ServerOperation("FIND_REPORT_ENTRIES")
  RecordListRequest<ReportRecord> findReportEntries(int firstResult, int maxResults);

  /**
   * @return a request object
   */
  @ServerOperation("FIND_REPORT_ENTRIES_BY_SEARCH")
  RecordListRequest<ReportRecord> findReportEntriesBySearch(Long employeeId,
      String department, String startsWith, String orderBy, int firstResult,
      int maxResults);

  /**
   * @return a request object
   */
  @ServerOperation("FIND_REPORTS_BY_EMPLOYEE")
  RecordListRequest<ReportRecord> findReportsByEmployee(
      PropertyReference<String> id);

}
